#include <Arduino.h>
#include <U8g2lib.h>
#include <BLEDevice.h>
#include <BLEUtils.h>
#include <BLEServer.h>

#ifdef U8X8_HAVE_HW_SPI
#include <SPI.h>
#endif
#ifdef U8X8_HAVE_HW_I2C
#include <Wire.h>
#endif

// BLE UUID
#define SERVICE_UUID        "4fafc201-1fb5-459e-8fcc-c5c9c331914b"
#define CHARACTERISTIC_UUID "beb5483e-36e1-4688-b7f5-ea07361b26a8"

// OLED
U8G2_SSD1306_128X64_NONAME_F_SW_I2C u8g2(U8G2_R0, /* clock=*/ SCL, /* data=*/ SDA, /* reset=*/ U8X8_PIN_NONE);

// Pressure pin
int sensorPin1 = A0;
int sensorPin2 = A1;
int sensorPin3 = A2;

// P and V
#define PRESS_MIN 20
#define PRESS_MAX 6000
#define VOLTAGE_MIN 0
#define VOLTAGE_MAX 5000

BLECharacteristic *pCharacteristic;
BLEServer *pServer;
BLEAdvertising *pAdvertising;

// BLE Server Callbacks
class MyServerCallbacks : public BLEServerCallbacks {
  void onConnect(BLEServer* pServer) {
    Serial.println("Client connected");
  }

  void onDisconnect(BLEServer* pServer) {
    Serial.println("Client disconnected");
    // Restart advertising
    pAdvertising->start();
    Serial.println("Restarted advertising");
  }
};

void setup() {
  Serial.begin(9600);
  analogReadResolution(12);  // ADC解析度为12位
  u8g2.begin();

  // 初始化BLE
  BLEDevice::init("CatHouse");
  pServer = BLEDevice::createServer();
  pServer->setCallbacks(new MyServerCallbacks());
  BLEService *pService = pServer->createService(SERVICE_UUID);
  pCharacteristic = pService->createCharacteristic(
                     CHARACTERISTIC_UUID,
                     BLECharacteristic::PROPERTY_READ |
                     BLECharacteristic::PROPERTY_WRITE |
                     BLECharacteristic::PROPERTY_NOTIFY
                   );
  pService->start();
  pAdvertising = pServer->getAdvertising();
  pAdvertising->start();
  Serial.println("Advertising started");
}

void loop() {
  long Fdata1 = getPressValue(sensorPin1);
  long Fdata2 = getPressValue(sensorPin2);
  long Fdata3 = getPressValue(sensorPin3);

  String data = String(Fdata1) + "," + String(Fdata2) + "," + String(Fdata3);
  pCharacteristic->setValue(data.c_str());
  pCharacteristic->notify();

  Serial.println(data);

  u8g2.clearBuffer();
  u8g2.setFont(u8g2_font_ncenB08_tr);

  u8g2.setCursor(0, 30);
  u8g2.print("P1: ");
  u8g2.print(Fdata1);

  u8g2.setCursor(0, 40);
  u8g2.print("P2: ");
  u8g2.print(Fdata2);

  u8g2.setCursor(0, 50);
  u8g2.print("P3: ");
  u8g2.print(Fdata3);

  u8g2.sendBuffer();
  
  delay(100);  // delay 100ms
}

long getPressValue(int pin) {
  long PRESS_AO = 0;
  int VOLTAGE_AO = 0;
  int value = analogRead(pin);

  VOLTAGE_AO = map(value, 0, 4095, 5000, 0);
  if (VOLTAGE_AO < VOLTAGE_MIN) {
    PRESS_AO = 0;
  } else if (VOLTAGE_AO > VOLTAGE_MAX) {
    PRESS_AO = PRESS_MAX;
  } else {
    PRESS_AO = map(VOLTAGE_AO, VOLTAGE_MIN, VOLTAGE_MAX, PRESS_MIN, PRESS_MAX);
  }
  
  return PRESS_AO;
}
